package com.ircstuff.jbounce.server;

import com.ircstuff.jbounce.config.*;

import java.io.*;
import java.net.*;

public class BouncerSession
{
	private static int sessionCount = 0;
	public synchronized static int createSessionId() {
		return sessionCount++;
	}

	private int sessionId = createSessionId();
	public BouncerSession(Socket socket) throws IOException
	{
		this.socket = socket;
		inin = socket.getInputStream();
		inout = socket.getOutputStream();
		new Thread(new ClientReader()).start();

		IRCBouncer.addSession(this);
		IRCBouncer.showSessions();
	}

	public String toString() {
		String u = user!=null?user.toString():"-unknown-";
		String result = "["+sessionId+"/"+u+"]";
		result += "from "+socket.getInetAddress().getHostName();
		if (outSock!=null)
			result += " to "+outSock.getInetAddress().getHostName();
		return result;
	}

	// reads all the input from the server
	private class ServerReader implements Runnable
	{
		public void run()
		{
			try {
				String line = null;
				while ((line=readLine(outin))!=null) sendRaw(inout, line);
			} catch(IOException e) {
				// session ended; proceed to shutdown
			}

			shutdown();
		}
	}

	// reads all the input from the client
	private class ClientReader implements Runnable
	{
		public void run()
		{
			try {

			String userString = null;
			String nickString = null;
			String passString = null;
			boolean registered = false;

			String line = null;
			while ((line=readLine(inin))!=null)
			{
				//System.out.println("*** ["+line+"]");
				if (!registered) {
					if (line.startsWith("USER")) {
						// read username
						userString = line;
					} else if (line.startsWith("NICK")) {
						// read nickname
						nickString = line;
						if (passString==null) {
							// if no password given yet, request one
							String nick = nickString.substring(5);
							sendRaw(inout, ":jbounce!jbounce@ircstuff.org NOTICE "+nick+" :Please do /QUOTE PASS yourpass to register");
						}
					} else if (line.startsWith("PASS")) {
						// read password
						passString = line;
					}

					// register the user when all the credentials are given
					if (userString!=null && nickString!=null && passString!=null) {
						boolean success = registerUser(userString, nickString, passString);
						if (success)
							registered = true;
						else
							return;
					}
				} else {
					sendRaw(outout, line);
				}
			}

			} catch(IOException e) { }
			//System.out.println("--- END (client)");

			shutdown();
		}
	}

	private boolean registerUser(String userString, String nickString, String passString) throws IOException {
		String userName = userString.substring(5, userString.indexOf(' ', 5));
		String realName = userString.substring(userString.indexOf(':')+1);
		String nick = nickString.substring(5);
		String pass = passString.substring(5);
		String customServer = null; 
		if (pass.indexOf('*')>=0) { // '*servername' may be prepended to the password
			customServer = pass.substring(pass.indexOf('*')+1);
			pass = pass.substring(0, pass.indexOf('*'));
		}

		// find a user that matches the connected user
		String mask = nick+"!"+userName+"@"+socket.getInetAddress().getHostName();
		user = IRCBouncer.getUser(mask, pass);

		// disconnect the client if he/she does not match one of our users
		if (user==null) {
			sendRaw(inout, "ERROR :Closing Link: ["+mask+"] (No access)");
			shutdown();
			return false;
		}

		System.out.println("customServer: "+customServer);
		System.out.println("defaultcon: "+user.getDefaultConnection());
		InetAddress inet = InetAddress.getByName(
			IRCBouncer.getServer(
				customServer==null?
				user.getDefaultConnection().getTo():
				user.getConnection(customServer).getTo()
			).getId()
		);
		//System.out.println("v6: ["+inet+"]");
		//InetAddress linet = InetAddress.getByName("3ffe:8114:2000:950::1");
		//InetAddress linet = InetAddress.getByName("3ffe:8114:1000::20b");
		InetAddress linet = InetAddress.getByName(
				customServer==null?
				user.getDefaultConnection().getFrom():
				user.getConnection(customServer).getFrom()
		);

		outSock = new Socket(inet, 6667, linet, 0);
		System.out.println("["+sessionId+"] connected");
		outin = outSock.getInputStream();
		outout = outSock.getOutputStream();
		new Thread(new ServerReader()).start();
		sendRaw(outout, userString);
		sendRaw(outout, nickString);
		System.out.println("["+sessionId+"] login data sent");

		return true;
	}

	private void shutdown() {
		try { socket.close(); } catch(IOException e) {}
		if (outSock != null)
			try { outSock.close(); } catch(IOException e) {}
		IRCBouncer.removeSession(this);
		IRCBouncer.showSessions();
	}

	protected static String readLine(InputStream in) throws IOException
	{
		StringBuffer result = new StringBuffer();
		int chr;
		while (((chr = in.read()) != -1) && (chr != 10) && (chr != 13))
			result.append((char) chr);
		if ((chr == -1) && (result.length() == 0))
			return null;
		if (chr == 13)
		{
			in.mark(1);
			if (in.read() != 10)
			in.reset();
		}
		return result.toString();
	}

	public void sendRaw(OutputStream dataOut, String message) throws IOException
	{
		dataOut.write((message+"\n").getBytes());
		dataOut.flush();
	}

	private InputStream inin, outin;
	private OutputStream inout, outout;
	private Socket socket, outSock;
	private User user;
}
