package com.ircstuff.jbounce.server;

import com.ircstuff.jbounce.config.*;

import java.io.*;
import java.net.*;
import java.util.*;

public class IRCBouncer implements Runnable
{
	public static void main(String[] argv)
	{
		String bouncerXML = argv.length>0?argv[0]:"bouncer.xml";
		File configFile = new File(bouncerXML);
		if (!configFile.exists()) {
			System.err.println("Configuration file '"+configFile.getName()+"' not found. Exiting.");
			System.exit(1);
		}

		Configuration config = new Configuration();
		new ConfigReader(config, configFile);
		users = config.getUsers();
		servers = config.getServers();
		for (Enumeration e=config.getListens().elements(); e.hasMoreElements();)
		{
			Listen listen = (Listen) e.nextElement();
			new IRCBouncer(listen.getIface(), listen.getPort());
		}
	}

	private static Vector servers, users;

	public static User getUser(String host, String pass)
	{
		for (Enumeration e=users.elements(); e.hasMoreElements();)
		{
			User user = (User) e.nextElement();
			if (user.matchesHost(host) && user.matchesPass(pass))
				return user;
		}

		return null;
	}

	public static Server getServer(String id)
	{
		System.out.println("finding server: ["+id+"]");
		for (Enumeration e=servers.elements(); e.hasMoreElements();)
		{
			Server server = (Server) e.nextElement();
			if (server.matches(id)) return server;
		}
		System.out.println("not found, server: ["+id+"]");

		return null;
	}

	public IRCBouncer(String localip, int localport)
	{
		System.out.println("Listening on "+localip+":"+localport);
		this.localip = localip;
		this.localport = localport;

		new Thread(this).start();
	}

	public void run()
	{
		try { runServer(); } catch(IOException e) { e.printStackTrace(); }
	}

	public void runServer() throws IOException
	{
		InetAddress localinet = InetAddress.getByName(localip);
		ServerSocket serverSocket = new ServerSocket(localport, 50, localinet);
		while (true)
		{
			Socket s = serverSocket.accept();
			new BouncerSession(s);
		}
	}

	private String localip;
	private int localport;

	private static Vector sessions = new Vector();
	public static void addSession(BouncerSession s) {
		sessions.add(s);
	}

	public static void removeSession(BouncerSession s) {
		sessions.remove(s);
	}

	public static void showSessions() {
		System.out.println("Number of active sessions: "+sessions.size());
		for (Enumeration e=sessions.elements(); e.hasMoreElements();)
		{
			BouncerSession s = (BouncerSession) e.nextElement();
			System.out.println("* "+s);
		}
	}
}
